unit uMain;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Forms, Controls, StdCtrls,

  Windows, uWinIpHlpApi, uWinIpTypes, uWinHelper;

type

  { TfrmMain }

  TfrmMain = class(TForm)
    btnInfo: TButton;
    Memo1: TMemo;
    procedure btnInfoClick(Sender: TObject);
  private

  public

  end;


var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

{ TfrmMain }

procedure TfrmMain.btnInfoClick(Sender: TObject);
const
  WORKING_BUFFER_SIZE = 15000;
  MAX_TRIES = 3;
var
  // Adapters
  pAddresses     : PIP_ADAPTER_ADDRESSES;
  pCurrAddresses : PIP_ADAPTER_ADDRESSES;
  pUnicast       : PIP_ADAPTER_UNICAST_ADDRESS_LH;
  pDnServer      : PIP_ADAPTER_DNS_SERVER_ADDRESS_XP;

  // Vars
  dwRetVal       : DWORD;
  outBufLen      : ULONG;
  i              : Integer;
  PhysicalAddress: String;

begin
  Memo1.Lines.Clear;

  // initialize
  pAddresses     := nil;
  pCurrAddresses := nil;
  pUnicast       := nil;
  pDnServer      := nil;
  dwRetVal       := 0;
  i              := 0;
  outBufLen      := 0;

  // -
  outBufLen := WORKING_BUFFER_SIZE;
  repeat
    if Assigned(pAddresses) then
      FreeMem(pAddresses);

    GetMem(pAddresses, outBufLen);
    if not Assigned(pAddresses) then
      raise Exception.Create('Memory allocation failed for IP_ADAPTER_ADDRESSES struct');

    dwRetVal := GetAdaptersAddresses(0, 0, nil, pAddresses, @outBufLen);
    Inc(i);
  until (dwRetVal <> ERROR_BUFFER_OVERFLOW) or (i = MAX_TRIES);

  // --
  try
    if NO_ERROR <> dwRetVal then begin
      if ERROR_NO_DATA = dwRetVal then begin
        Memo1.Lines.Add('Error');
        Exit;
      end
      else
        raise Exception.Create(SysErrorMessage(dwRetVal));
    end;

    pCurrAddresses := pAddresses;
    Memo1.Lines.BeginUpdate;
    while Assigned(pCurrAddresses) do begin
      if pCurrAddresses^.PhysicalAddressLength > 0 then begin

        memo1.Lines.Add( 'AdapterName: ' +#9#9+ pCurrAddresses^.AdapterName );

        // Unicast
        pUnicast := pCurrAddresses^.FirstUnicastAddress;
        if (pUnicast <> nil) then begin
          while (pUnicast <> nil) do begin
            if (pUnicast^.Address.lpSockAddr^.sa_family = AF_INET) then begin
              // IPv4
              memo1.Lines.Add( 'IPv4 Address: ' +#9#9+ SockAddr4ToStr(pUnicast^.Address) );

              // Broadcast
              memo1.Lines.Add( 'Broadcast: ' +#9#9+ '?' );

              // NetMask                                    // ~ 192.168.0.1/24
              memo1.Lines.Add( 'NetMask: ' +#9#9#9+ pUnicast^.OnLinkPrefixLength.ToString );
            end;

            // IPv6
            if (pUnicast^.Address.lpSockAddr^.sa_family = AF_INET6) then begin
              memo1.Lines.Add( 'IPv6 Address: ' +#9#9+ 'no IPv6 support in socketsh.inc' );
            end;

            pUnicast := pUnicast^.Next;
          end;
        end;

        memo1.Lines.Add( 'DNS Suffix: ' +#9#9+ pCurrAddresses^.DnsSuffix );
        memo1.Lines.Add( 'Description: ' +#9#9+ pCurrAddresses^.Description );
        memo1.Lines.Add( 'FriendlyName: ' +#9#9+ pCurrAddresses^.FriendlyName );

        // Physical address
        if (pCurrAddresses^.PhysicalAddressLength <> 0) then begin
          PhysicalAddress := '';
          for i := 0 to pCurrAddresses^.PhysicalAddressLength - 1 do begin
            if (i = (pCurrAddresses^.PhysicalAddressLength - 1)) then
              PhysicalAddress := PhysicalAddress + Format('%.2x', [pCurrAddresses^.PhysicalAddress[i]])
            else
              PhysicalAddress := PhysicalAddress + Format('%.2x-', [pCurrAddresses^.PhysicalAddress[i]]);
          end;
        end;
        memo1.Lines.Add( 'Physical address: ' +#9#9+ PhysicalAddress );

        // DNS
        pDnServer := pCurrAddresses^.FirstDnsServerAddress;
        if (pDnServer <> nil) then begin
          while (pDnServer <> nil) do begin
            memo1.Lines.Add( 'DNS Server: ' +#9#9+ SockAddr4ToStr(pDnServer^.Address) );
            pDnServer := pDnServer^.Next;
          end;
        end;

        // DHCP
        memo1.Lines.Add( 'DHCP Server: ' + #9#9 + SockAddr4ToStr(pCurrAddresses^.DHCPv4Server) );

        // Flags
        memo1.Lines.Add( 'Flags: ' );
        memo1.Lines.Add( ' -  DdnsEnabled: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.DdnsEnabled, True) );
        memo1.Lines.Add( ' -  Dhcpv4Enabled: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.Dhcpv4Enabled, True) );
        memo1.Lines.Add( ' -  Ipv4Enabled: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.Ipv4Enabled, True) );
        memo1.Lines.Add( ' -  Ipv6Enabled: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.Ipv6Enabled, True) );
        memo1.Lines.Add( ' -  Ipv6ManagedSupported: ' +#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.Ipv6ManagedAddressConfigurationSupported, True) );
        memo1.Lines.Add( ' -  Ipv6OtherStatefulConfig: ' +#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.Ipv6OtherStatefulConfig, True) );
        memo1.Lines.Add( ' -  NetbiosOverTcpipEnabled: ' +#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.NetbiosOverTcpipEnabled, True) );
        memo1.Lines.Add( ' -  NoMulticast: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.NoMulticast, True) );
        memo1.Lines.Add( ' -  ReceiveOnly: ' +#9#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.ReceiveOnly, True) );
        memo1.Lines.Add( ' -  RegisterAdapterSuffix: ' +#9+ BoolToStr(pCurrAddresses^.Flags.BitFlags.RegisterAdapterSuffix, True) );


        memo1.Lines.Add( 'Mtu: ' + #9#9#9 + pCurrAddresses^.Mtu.ToString );
        memo1.Lines.Add( 'IfType: ' + #9#9#9 + pCurrAddresses^.IfType.ToString );

        Memo1.Lines.Add('-------------------------------------------');
        Memo1.Lines.Add('');
      end;
      pCurrAddresses := pCurrAddresses^.Next;
    end;
    Memo1.Lines.EndUpdate;

  finally
    if Assigned(pAddresses) then
      FreeMem(pAddresses);
  end;

end;



end.

